﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;

//====================================================
// Описание работы классов и методов исходника на:
// https://www.interestprograms.ru
// Исходные коды программ и игр
//====================================================

namespace WpfMoveWindow
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            GeometryFormWindow(myGrid);
        }

        #region Геометрическая фигура в роли формы окна

        void GeometryFormWindow(Panel panel)
        {
            // Создание комплексной фигуры.
            GeometryGroup geometryGroup = new();

            int count = 7;
            double angle = 360.0 / count;
            double radius = 210;
            for (int i = 0; i < count; i++)
            {
                double tmp = angle * i;
                double rad = tmp * Math.PI / 180.0;

                double x = radius * Math.Cos(rad) + Width / 2;
                double y = radius * Math.Sin(rad) + Height / 2;

                geometryGroup.Children.Add
                (
                    new EllipseGeometry()
                    {
                        Center = new Point(x, y),
                        RadiusX = 50,
                        RadiusY = 50,
                    }
                );
            }

            EllipseGeometry eCenter = new()
            {
                Center = new Point(400, 400),
                RadiusX = 150,
                RadiusY = 150
            };
            geometryGroup.Children.Add((eCenter));

            Path figureWindow = new()
            {
                Data = geometryGroup,
                Fill = Brushes.DarkGoldenrod,
                StrokeThickness = 0,
                HorizontalAlignment = HorizontalAlignment.Left,
                VerticalAlignment = VerticalAlignment.Top
            };

            panel.Children.Add(figureWindow);
        }

        #endregion


        #region Функциональность перемещения окна

        private bool _canMove = false;
        private Point _offsetPoint = new(0, 0);
        private void Window_MouseDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            Point p = e.MouseDevice.GetPosition(this);
            _canMove = true;


            // --- Первый способ вычисления смещения указателя мыши ---

            _offsetPoint.X = PointToScreen(p).X - this.Left;
            _offsetPoint.Y = PointToScreen(p).Y - this.Top;

            // --- / ---


            // --- Второй способ вычисления смещения указателя мыши ---

            /* // Значения корректировки смещения при различных стилях окна.
             double frameThickness = (this.ActualWidth - myGrid.ActualWidth) / 2;
             double captionHeight = (this.ActualHeight - myGrid.ActualHeight) - frameThickness;


             // Позиция курсора относительно данного окна.
             _offsetPoint.Y = p.Y + captionHeight;
             _offsetPoint.X = p.X + frameThickness;

             // Диагностика
             System.Diagnostics.Debug.WriteLine("frameThickness=" + frameThickness);
             System.Diagnostics.Debug.WriteLine("captionHeight=" + captionHeight);
             // --- / ---*/


            e.MouseDevice.Capture(this);
        }

        private void Window_MouseMove(object sender, System.Windows.Input.MouseEventArgs e)
        {
            if (_canMove == true)
            {
                Point p = e.MouseDevice.GetPosition(this);

                Left = PointToScreen(p).X - _offsetPoint.X;
                Top = PointToScreen(p).Y - _offsetPoint.Y;
            }
        }

        private void Window_MouseUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            _canMove = false;
            e.MouseDevice.Capture(null);
        }

        #endregion
    }
}
